/*--------------------------------------------------------------------
 * kb_R900.h - KoreBot Library - R900 functions
 *--------------------------------------------------------------------
 * $Id: kb_R900.h,v 1.3 2006/08/24 12:33:46 flambercy Exp $
 *--------------------------------------------------------------------
 * $Author: flambercy $
 * $Date: 2008/09/11 12:33:46 $
 * $Revision: 1.0 $
 *-------------------------------------------------------------------*/ 

#ifndef __r900__
#define __r900__

#include "knet.h"

/*!
 *  R900 <-> Korebot gpios pin definitions
 */
#define IN               1
#define OUT		 0


/*! 
 *  R900 Error codes
 */ 
#define R900_ERROR_SUCCESS	0		/* No errors */
#define R900_ERROR_KBINIT	-1		/* Unable to initialize the korebot library */
#define R900_ERROR_FRMSZ   	-2		/* Frame received doesnt have the expected size */
#define R900_ERROR_SZFMT   	-3		/* Frame size received should be 1 byte wide, wrong format */
#define R900_ERROR_FRMSND  	-4		/* Unable to send a command frame to the R900 */

/*!
 * R900 command delay
 */
#define R900_CMD_DELAY	    300



/*!--------------------------------------------------------------------
 * Prototypes Declaration
 */
/* Need to be called before any other functions */
extern int r900_init( void );

/* "Low level" function to communicate with the R900 via i2c */
extern int r900_getcommand( knet_dev_t * , unsigned char *);
extern int r900_sendcommand( knet_dev_t *, unsigned char *);

/* "High level" function that let user to simply retrieves various informations from the robot */
extern int r900_proximity_ir(char *, knet_dev_t *);
extern int r900_ambiant_ir(char *, knet_dev_t *);
extern int r900_battery_voltage(char *, knet_dev_t *);
extern int r900_Read_TV(char *, knet_dev_t *);
extern int r900_Sound_Event(char *, knet_dev_t *);
extern int r900_reset_tstamp(char *, knet_dev_t *);
extern int r900_revision(char *, knet_dev_t *);
extern int r900_setLed(char *, knet_dev_t *);
extern int r900_setBackupLight(char *, knet_dev_t *);
extern int r900_korebot_disable(char *, knet_dev_t *);
extern int r900_set_motor(char *, knet_dev_t *);
extern int r900_koremotor_disable(char *, knet_dev_t *);
extern int r900_Enable_modem(char *,unsigned char , knet_dev_t *);
extern int r900_Send_SMS(char *,unsigned char , char *, knet_dev_t *);
extern int r900_Set_Phone_Number(char *,unsigned char , char *, knet_dev_t *);
	


#endif /* __R900__ */
