/*! 
 * \file   khepera3_test.c  Khepera3 test application              
 *
 * \brief 
 *         This is an application that demonstrates the various R900 commands. 
 *         
 *        
 * \author   Frederic Lambercy (K-Team SA)                               
 *
 * \note     Copyright (C) 2008 K-TEAM SA
 * \bug      none discovered.                                         
 * \todo     nothing.
 */

#include <korebot/korebot.h>

#define MAXBUFFERSIZE 100

static int quitReq = 0;
static char buf[1024];


/*! handle to the various r900 devices (knet socket, i2c mode)
 */
static knet_dev_t * dsPic;



/*--------------------------------------------------------------------*/
/*! initR900 initialize various things in the R900 
 *
 * \return A value :
 *      - 1 if success
 *      - 0 if any error
 */
int initR900( void )
{
  R900_init();
  
  /* open various socket and store the handle in their respective pointers */
  dsPic = knet_open( "R900:dsPic" , KNET_BUS_I2C , 0 , NULL );

  if(dsPic!=0)
  {
      return 0;
  }


} 


/*--------------------------------------------------------------------*/
/*! proxIR retrieves proximity ir measure using kb_r900.c library.
 */
int proxIR( int argc, char * argv[], void * data)
{
  char Buffer[MAXBUFFERSIZE];
  if(r900_proximity_ir((char *)Buffer, dsPic))
  	printf("\r\n%c,%4.4u,%4.4u,%4.4u,%4.4u,%4.4u,%4.4u,%4.4u,%lu\r\n",
	   Buffer[0], (Buffer[1] | Buffer[2]<<8), (Buffer[3] | Buffer[4]<<8),
	   (Buffer[5] | Buffer[6]<<8), (Buffer[7] | Buffer[8]<<8),
	   (Buffer[9] | Buffer[10]<<8), (Buffer[11] | Buffer[12]<<8),
	   (Buffer[13] | Buffer[14]<<8), 
	   ((Buffer[15] | Buffer[16]<<8) | (Buffer[17] | Buffer[18]<<8)<<16));
  else
  	printf("\r\nn, error...\r\n");
}
		
/*--------------------------------------------------------------------*/
/*! ambIR retrieves ambiant ir measure using kb_r900.c library.
 */
int ambIR( int argc, char * argv[], void * data)
{
  char Buffer[MAXBUFFERSIZE];
  if(r900_ambiant_ir((char *)Buffer, dsPic))
   	printf("\r\n%c,%4.4u,%4.4u,%lu\r\n",
	   Buffer[0], (Buffer[1] | Buffer[2]<<8), (Buffer[3] | Buffer[4]<<8),
	   ((Buffer[5] | Buffer[6]<<8) | (Buffer[7] | Buffer[8]<<8)<<16));
  else
  	printf("\r\no, error...\r\n");
}

/*--------------------------------------------------------------------*/
/*! voltageBAT retrieves the battery voltage using kb_r900.c library.
 */
int voltageBAT( int argc, char * argv[], void * data)
{
  char Buffer[MAXBUFFERSIZE];
  short argument;
  float Voltage;

  if(r900_battery_voltage((char *)Buffer, dsPic)){
	Voltage = (Buffer[1] | Buffer[2]<<8) / 112.809;
 	printf("\r\n%c, battery voltage = %3.3fV\r\n",
	  Buffer[0], Voltage );
  }
  else
	  printf("\r\nv, error...\r\n");
}

/*--------------------------------------------------------------------*/
/*! ReadTV retrieves the message coming from the TV receiver using kb_r900.c library.
 */
int ReadTV( int argc, char * argv[], void * data)
{
  char Buffer[MAXBUFFERSIZE];
  short argument;
  float Voltage;

  if(r900_Read_TV((char *)Buffer, dsPic)){
        printf("\r\n%c, TV message = %d, Time = %lu\r\n", Buffer[0], Buffer[1], ((Buffer[2] | Buffer[3]<<8) | (Buffer[4] | Buffer[5]<<8)<<16) );
  }
  else
          printf("\r\nv, error...\r\n");
}


/*--------------------------------------------------------------------*/
/*! Sound retrieves sound event using kb_r900.c library.
 */
int Sound( int argc, char * argv[], void * data)
{
  char Buffer[MAXBUFFERSIZE];
  short argument;
  float Voltage;

  if(r900_Sound_Event((char *)Buffer, dsPic)){
        printf("\r\n%c, Micro %d, Volume = %d, Time = %lu\r\n", Buffer[0], Buffer[1], Buffer[2] | (Buffer[3]<<8),
		 ((Buffer[4] | Buffer[5]<<8) | (Buffer[6] | Buffer[7]<<8)<<16) );
  }
  else
          printf("\r\nv, error...\r\n");
}



/*--------------------------------------------------------------------*/
/*! tstampRST resets the relative time stamp using kb_r900.c library.
 */
int tstampRST( int argc, char * argv[], void * data)
{
  char Buffer[MAXBUFFERSIZE];

  if(r900_reset_tstamp((char *)Buffer, dsPic))
  	printf("\r\n%c\r\n", Buffer[0]);
  else
	printf("\r\nz, error...\r\n");
}

/*--------------------------------------------------------------------*/
/*! revisionOS retrieves the R900 os version using kb_R900.c library.
 */
int revisionOS( int argc, char * argv[], void * data)
{
  char Buffer[MAXBUFFERSIZE];	/* buffer that handle the returned datas from kh3 */
  unsigned int ver ;




  if(r900_revision((char *)Buffer, dsPic)){
  	printf("\r\n%c,%4.4u,%4.4u => Version = %u, Revision = %u\r\n", 
	   Buffer[0], (Buffer[1] | Buffer[2]<<8), (Buffer[3] | Buffer[4]<<8),
	   (Buffer[1] | Buffer[2]<<8), (Buffer[3] | Buffer[4]<<8));
/* kmot_GetFWVersion( mot1 , &ver );
    printf("Right Motor Firmware v%u.%u\n" , 
        KMOT_VERSION(ver) , KMOT_REVISION(ver) );

   kmot_GetFWVersion( mot2 , &ver );
    printf("Left Motor Firmware v%u.%u\n" , 
        KMOT_VERSION(ver) , KMOT_REVISION(ver) );
*/

  }
  else
	printf("\r\nb, error...\r\n");  
}


/*--------------------------------------------------------------------*/
/*! EnableModem enable/init or disable the modem  using kb_R900.c library.
 */
int EnableModem(int argc, char * argv[], void * data)
{
  char Buffer[MAXBUFFERSIZE];

  if(r900_Enable_modem((char *)Buffer,atoi(argv[1]),  dsPic))
        printf("\r\n%c\r\n", Buffer[0]);
  else
        printf("\r\nz, error...\r\n");
}

/*--------------------------------------------------------------------*/
/*! SetPhone set the phone number of the recipient using kb_R900.c library.
 */
int SetPhone(int argc, char * argv[], void * data)
{
  char Buffer[MAXBUFFERSIZE];
  char size, i;
  char Number[16];

  for(i = 0, size = 0; (*(argv[1]+i) != '\r') && (*(argv[1]+i) != '\n') && (*(argv[1]+i) != '\0');   i++, size++)
  {
	Number[i] = *(argv[1] + i);
  }

  if(r900_Set_Phone_Number((char *)Buffer,size, (char *)Number,  dsPic))
        printf("\r\n%c\r\n", Buffer[0]);
  else
        printf("\r\nz, error...\r\n");

}

/*--------------------------------------------------------------------*/
/*! SendSMS send a SMS text with the GSM kb_R900.c library.
 */
int SendSMS(int argc, char * argv[], void * data)
{
  char Buffer[MAXBUFFERSIZE];
  char size, i;
  char Text[163];

  for(i = 0, size = 0; (*(argv[1]+i) != '\r') && (*(argv[1]+i) != '\n') && (*(argv[1]+i) != '\0');   i++, size++)
  {
        
	Text[i] = *(argv[1]+i);

  }

  if(r900_Send_SMS((char *)Buffer,size, (char *)Text,  dsPic))
        printf("\r\n%c\r\n", Buffer[0]);
  else
        printf("\r\nz, error...\r\n");
}


/*--------------------------------------------------------------------*/
/*! setMotor set the speed of the motor to mount or to descend the platform using kb_R900.c library.
 */

int setMotor(int argc, char * argv[], void * data)
{
  char Buffer[MAXBUFFERSIZE];

  if(r900_set_motor((char *)Buffer,atoi(argv[1]), atoi(argv[2]),  dsPic))
        printf("\r\n%c\r\n", Buffer[0]);
  else
        printf("\r\nz, error...\r\n");

}

/*--------------------------------------------------------------------*/
/*! setLED set the state of the 4 LED of the R900 using kb_R900.c library.
 */

int setLED(int argc, char * argv[], void * data)
{
  char Buffer[MAXBUFFERSIZE];

  if(r900_setLed((char *)Buffer,atoi(argv[1]), atoi(argv[2]),  dsPic))
        printf("\r\n%c\r\n", Buffer[0]);
  else
        printf("\r\nz, error...\r\n");

}


/*--------------------------------------------------------------------*/
/*! setLight set the state of the backup light using kb_R900.c library.
 */

int setLight(int argc, char * argv[], void * data)
{
  char Buffer[MAXBUFFERSIZE];

  if(r900_setBackupLight((char *)Buffer,atoi(argv[1]),  dsPic))
        printf("\r\n%c\r\n", Buffer[0]);
  else
        printf("\r\nz, error...\r\n");

}

/*--------------------------------------------------------------------*/
/*! DisableKBOT cut the supply of the KoreBot kb_R900.c library.
 */

int disableKBOT(int argc, char * argv[], void * data)
{
  char Buffer[MAXBUFFERSIZE];

  if(r900_korebot_disable((char *)Buffer,  dsPic))
        printf("\r\n%c\r\n", Buffer[0]);
  else
        printf("\r\nz, error...\r\n");

}

/*--------------------------------------------------------------------*/
/*! enableKMOT turn on or off the supply of the KMot motors kb_R900.c library.
 */

int enableKMOT(int argc, char * argv[], void * data)
{
  char Buffer[MAXBUFFERSIZE];

  if(r900_koremotor_disable((char *)Buffer, atoi(argv[1]),  dsPic))
        printf("\r\n%c\r\n", Buffer[0]);
  else
        printf("\r\nz, error...\r\n");

}


#if 0
/*--------------------------------------------------------------------*/
/*! motSpeed configures the motor controller speed in the engine control unit.
 *
 *  \param 1st first param (argv[1]) is the motor1 speed.
 *  \param 2nd second param (argv[2]) is the motor2 speed.
 *
 */
int motSpeed( int argc, char *argv[], void *data)
{
  if(mot1!=0 && mot2!=0)
  {
  	kmot_SetPoint( mot1 , kMotRegSpeed , atoi(argv[1]));
  	kmot_SetPoint( mot2 , kMotRegSpeed , atoi(argv[2]));
	return 0;
  }
  else
	return -1;
}

/*--------------------------------------------------------------------*/
/*! motMove configures the motor controller position in the engine control unit.
 *
 *  \param 1st first param (argv[1]) is the motor1 position.
 *  \param 2nd second param (argv[2]) is the motor2 position.
 *
 */
int motMove( int argc, char *argv[], void *data)
{
  if(mot1!=0 && mot2!=0)
  {
	kmot_SetPoint( mot1 , kMotRegPosProfile, atoi(argv[1]));
  	kmot_SetPoint( mot2 , kMotRegPosProfile, atoi(argv[2]));
	return 0;
  }
  else
	return -1;
}

/*--------------------------------------------------------------------*/
/*! motStop stops the motor  in the engine control unit.
 *
 *  \param none.
 */
int motStop( int argc, char *argv[], void *data)
{
  if(mot1!=0 && mot2!=0)
  {
	kmot_SetMode( mot1 , kMotModeStopMotor );
	kmot_SetMode( mot2 , kMotModeStopMotor );
	return 0;
  }
  else
	return -1;
}
	  
/*--------------------------------------------------------------------*/
/*! Braintenberg demo program
 *
 *  \param none.
 */

#define BR_IRGAIN 30
#define fwSpeed 20
#define MIN_SPEED 10
#define RotSpeedL 50
#define RotSpeedR -50
 
int braitenbergAvoidance( int argc , char * argv[] , void * data)
{
  int Connections_B[9] = { 2, -2, -4, -12,  5,  2,  2, 2, 4}; // weight of every 9 sensor for the left motor 
  int Connections_A[9] = { 2,  2,  2, 5, -12, -4, -2, 2, 4}; // weight of every 9 sensor for the right motor 
  int i, buflen, sensval;
  char buffer[MAXBUFFERSIZE];
  char * scan;
  long int lspeed16, rspeed16;
  int tabsens[9];
  int left_speed, right_speed;
  unsigned int immobility = 0;
  unsigned int prevpos_left, pos_left, prevpos_right,  pos_right;
  u_int8_t valueLL,valueLH,valueHL,valueHH;


  char cmd[3] = { 2, 'N', 0};

  /* Get the current position values */
  prevpos_left = kmot_GetMeasure( mot1 , kMotRegPos );
  prevpos_right = kmot_GetMeasure( mot2 , kMotRegPos );

  while(1)
  {
    lspeed16 = 0; rspeed16 = 0;

#if 0
    kh3_sendcommand( dsPic, cmd );
    while(!kb_gpio_get(KNET_INT0));

    buflen = knet_llread( dsPic, buffer, 30);
#endif
    kh3_proximity_ir((char *)buffer, dsPic);
    scan   = buffer+3;

    /* limit the sensor values to 0-max */
    for (i = 0; i < 9; i++)	
    {
	sensval = *(scan) | (*(scan+1))<<8;
	if(sensval > 1000)
	  tabsens[i] = 450;
	else if (sensval < 100)
	  tabsens[i] = 0;
	else
	  tabsens[i] = (sensval - 100) >> 1;
	scan = scan + 2;
    }

    for (i = 0; i < 9; i++)
    {
      lspeed16 += Connections_A[i] * tabsens[i];
      rspeed16 += Connections_B[i] * tabsens[i];				
    }

    left_speed = ((lspeed16 / BR_IRGAIN) + fwSpeed);
    right_speed = ((rspeed16 / BR_IRGAIN) + fwSpeed);

    if(left_speed > 0 && left_speed < MIN_SPEED)
      left_speed = MIN_SPEED;
    if(left_speed < 0 && left_speed > -MIN_SPEED)
      left_speed = -MIN_SPEED;
    if(right_speed > 0 && right_speed < MIN_SPEED)
      right_speed = MIN_SPEED;
    if(right_speed < 0 && right_speed > -MIN_SPEED)
      right_speed = -MIN_SPEED;

/* define Shift_speed for version under 1.4*/
#ifndef  SHIFT_SPEED
     left_speed *= 256;
     right_speed *= 256;
#endif


    kmot_SetPoint( mot1, kMotRegSpeed, left_speed);
    kmot_SetPoint( mot2, kMotRegSpeed, right_speed);

    printf("lens = %d, rsens = %d lspd = %d rspd = %d\r\n", (int)lspeed16, (int)rspeed16, left_speed, right_speed);	

    left_speed = kmot_GetMeasure( mot1 , kMotMesSpeed );
    right_speed = kmot_GetMeasure( mot2 , kMotMesSpeed );

    /* Get the new position of the wheel to compare with previous values */
    pos_left = kmot_GetMeasure( mot1 , kMotRegPos );
    pos_right = kmot_GetMeasure( mot2 , kMotRegPos );


    if((pos_left < (prevpos_left + 700)) && (pos_left > (prevpos_left -700)) && (pos_right < (prevpos_right + 700)) && (pos_right > (prevpos_right -700)))
    {
      if(++immobility > 5)
      {
         left_speed = RotSpeedL;
         right_speed = RotSpeedR;
#ifndef  SHIFT_SPEED
         left_speed *= 256;
         right_speed *= 256;
#endif
         kmot_SetPoint( mot1, kMotRegSpeed, left_speed);
         kmot_SetPoint( mot2, kMotRegSpeed, right_speed);

	 do{
	    usleep(10);
	    kh3_proximity_ir((char *)buffer, dsPic);
         }while (((buffer[7] | (buffer[8]<<8) ) >250) || ((buffer[9] | (buffer[10]<<8)) >250));

         immobility = 0;
         prevpos_left = pos_left;
         prevpos_right = pos_right;
      }
    }
    else
    {
       immobility = 0;
       prevpos_left = pos_left;
       prevpos_right = pos_right;
    }


    printf("lspd = %d rspd = %d\r\n", left_speed, right_speed); 			

    usleep(20000); 
  }
}
#endif

/*--------------------------------------------------------------------*/
/*! Quit the program.
*/
int quit( int argc , char * argv[] , void * data)
{
  quitReq = 1;
}


int help( int argc , char * argv[] , void * data);
/*--------------------------------------------------------------------*/
/*! The command table contains:
 * command name : min number of args : max number of args : the function to call
 */
static kb_command_t cmds[] = {
 { "quit"            , 0 , 0 , quit } ,
 { "exit"            , 0 , 0 , quit } ,
 { "bye"             , 0 , 0 , quit } ,
 { "A"		     , 1 , 1 , SendSMS },
 { "B"		     , 0 , 0 , revisionOS },
 { "C"		     , 1 , 1 , SetPhone },
 { "D"		     , 1 , 1 , EnableModem },
 { "V"		     , 0 , 0 , voltageBAT },
 { "Z"		     , 0 , 0 , tstampRST },    
 { "O"		     , 0 , 0 , ambIR },
 { "N"		     , 0 , 0 , proxIR },
 { "M"               , 2 , 2 , setMotor },
 { "L"		     , 2 , 2 , setLED },
 { "E"		     , 1 , 1 , setLight },
 { "P"		     , 1 , 1 , enableKMOT },
 { "K"		     , 0 , 0 , disableKBOT },
 { "S"		     , 0 , 0 , Sound },
 { "T"		     , 0 , 0 , ReadTV },
// { "setmotspeed"     , 2 , 2 , motSpeed },
// { "setmotmove"      , 2 , 2 , motMove },
// { "motstop"         , 0 , 0 , motStop },
// { "braiten"         , 0 , 0 , braitenbergAvoidance},
 { "help"            , 0 , 0 , help } ,
 { NULL              , 0 , 0 , NULL }
};

/*--------------------------------------------------------------------*/
/*! Display a list of available commands.
*/
int help( int argc , char * argv[] , void * data)
{
  kb_command_t * scan = cmds;
  while(scan->name != NULL)
  {
    printf("%s\r\n",scan->name);
    scan++;
  }
  return 0;
}

int main( int arc, char *argv[])
{
  char Buffer[MAXBUFFERSIZE];
  char i;


  printf("R900 test program (C) K-Team S.A\r\n");

  if(!initR900())
  {
    printf("Init ok...\r\n");


    while (!quitReq) 
    {


      printf("\n> ");

      if ( fgets( buf , sizeof(buf) , stdin ) != NULL ) 
      {
	buf[strlen(buf)-1] = '\0';
	kb_parse_command( buf , cmds , NULL);
      }
    }

    printf("Exiting...\r\n");
	}
	else
	  printf("Fatal error, unable to initialize\r\n");

}


